@@ -10,6 +10,7 @@ extern "C" {
 #include <map>
 #include <memory>
 #include <vector>
+#include <list>
 
 //////////////////////////////
 extern ProcList *processes;
@@ -33,7 +34,7 @@ static fd_set pc_loop_fd_set;
 static std::vector<int> pc_loop_fd_list;
 static bool pc_loop_use_select = true;
 
-static handle *handles = NULL;
+static std::list<handle> handles;
 
 static std::pair<int, int> create_self_pipe() {
   int pfd[2];
@@ -117,7 +118,7 @@ static int nethogsmonitor_init(int devc, char **devicenames, bool all,
       if (dp_setnonblock(newhandle, 1, errbuf) == -1) {
         fprintf(stderr, "Error putting libpcap in nonblocking mode\n");
       }
-      handles = new handle(newhandle, current_dev->name, handles);
+      handles.push_front(handle(newhandle, current_dev->name));
 
       if (pc_loop_use_select) {
         // some devices may not support pcap_get_selectable_fd
@@ -207,10 +208,14 @@ static void nethogsmonitor_handle_update(NethogsMonitorCallback cb) {
       const u_int32_t uid = curproc->getVal()->getUid();
       u_int64_t sent_bytes;
       u_int64_t recv_bytes;
+      u_int64_t sent_bytes_last;
+      u_int64_t recv_bytes_last;
+
       float sent_kbs;
       float recv_kbs;
       curproc->getVal()->getkbps(&recv_kbs, &sent_kbs);
       curproc->getVal()->gettotal(&recv_bytes, &sent_bytes);
+      curproc->getVal()->getlast(&recv_bytes_last, &sent_bytes_last);
 
       // notify update
       bool const new_data =
@@ -239,9 +244,12 @@ static void nethogsmonitor_handle_update(NethogsMonitorCallback cb) {
       NHM_UPDATE_ONE_FIELD(data.uid, uid)
       NHM_UPDATE_ONE_FIELD(data.sent_bytes, sent_bytes)
       NHM_UPDATE_ONE_FIELD(data.recv_bytes, recv_bytes)
+      NHM_UPDATE_ONE_FIELD(data.sent_bytes_last, sent_bytes_last)
+      NHM_UPDATE_ONE_FIELD(data.recv_bytes_last, recv_bytes_last)
       NHM_UPDATE_ONE_FIELD(data.sent_kbs, sent_kbs)
       NHM_UPDATE_ONE_FIELD(data.recv_kbs, recv_kbs)
 
+
 #undef NHM_UPDATE_ONE_FIELD
 
       if (data_change) {
@@ -257,15 +265,10 @@ static void nethogsmonitor_handle_update(NethogsMonitorCallback cb) {
 
 static void nethogsmonitor_clean_up() {
   // clean up
-  handle *current_handle = handles;
-  handle *rem;
-  while (current_handle != NULL) {
+  for(auto current_handle = handles.begin(); current_handle != handles.end(); current_handle++){
     pcap_close(current_handle->content->pcap_handle);
-    rem = current_handle;
-    current_handle = current_handle->next;
-    free(rem);
   }
-  handles = NULL;
+  handles.clear();
 
   // close file descriptors
   for (std::vector<int>::const_iterator it = pc_loop_fd_list.begin();
@@ -277,11 +280,12 @@ static void nethogsmonitor_clean_up() {
 }
 
 int nethogsmonitor_loop(NethogsMonitorCallback cb, char *filter, int to_ms) {
-    return nethogsmonitor_loop_devices(cb, filter, 0, NULL, false, to_ms);
+  return nethogsmonitor_loop_devices(cb, filter, 0, NULL, false, to_ms);
 }
 
 int nethogsmonitor_loop_devices(NethogsMonitorCallback cb, char *filter,
-                                int devc, char **devicenames, bool all, int to_ms) {
+                                int devc, char **devicenames, bool all,
+                                int to_ms) {
   if (monitor_run_flag) {
     return NETHOGS_STATUS_FAILURE;
   }
@@ -299,8 +303,7 @@ int nethogsmonitor_loop_devices(NethogsMonitorCallback cb, char *filter,
   while (monitor_run_flag) {
     bool packets_read = false;
 
-    handle *current_handle = handles;
-    while (current_handle != NULL) {
+    for(auto current_handle = handles.begin(); current_handle != handles.end(); current_handle++) {
       userdata->device = current_handle->devicename;
       userdata->sa_family = AF_UNSPEC;
       int retval = dp_dispatch(current_handle->content, -1, (u_char *)userdata,
@@ -312,7 +315,6 @@ int nethogsmonitor_loop_devices(NethogsMonitorCallback cb, char *filter,
       } else {
         gettimeofday(&curtime, NULL);
       }
-      current_handle = current_handle->next;
     }
 
     time_t const now = ::time(NULL);
@@ -337,3 +339,25 @@ void nethogsmonitor_breakloop() {
   monitor_run_flag = false;
   write(self_pipe.second, "x", 1);
 }
+
+void nethogs_packet_stats(NethogsPackageStats **stats, int *stats_size)
+{
+  
+  *stats = static_cast<NethogsPackageStats *>(malloc(handles.size() * sizeof(NethogsPackageStats)));
+  int i = 0;
+
+  for(auto current_handle = handles.begin(); current_handle != handles.end(); current_handle ++){
+    dp_stat stat = dp_stats(current_handle->content);
+    stats[i]->ps_recv = stat.ps_recv;
+    stats[i]->ps_drop = stat.ps_drop;
+    stats[i]->ps_ifdrop = stat.ps_ifdrop;
+    stats[i]->devicename = current_handle->devicename;
+    i++;
+  }
+  *stats_size = handles.size();
+}
+
+void nethogs_enable_udp(bool state)
+{
+  catchall = state;
+}
\ No newline at end of file
