@@ -7,6 +7,7 @@ extern "C" {
 
 #include <stdbool.h>
 #include <stdint.h>
+#include <sys/types.h>
 
 #define NETHOGS_DSO_VISIBLE __attribute__((visibility("default")))
 #define NETHOGS_DSO_HIDDEN __attribute__((visibility("hidden")))
@@ -26,10 +27,20 @@ typedef struct NethogsMonitorRecord {
   const char *device_name;
   uint64_t sent_bytes;
   uint64_t recv_bytes;
+  uint64_t sent_bytes_last;
+  uint64_t recv_bytes_last;
   float sent_kbs;
   float recv_kbs;
 } NethogsMonitorRecord;
 
+typedef struct NethogsPackageStats
+{
+  u_int ps_recv;           /** number of packets received */
+  u_int ps_drop;           /** number of packets dropped because there was no room in the operating system's buffer when they arrived, because packets weren't being read fast enough */
+  u_int ps_ifdrop;         /** number of packets dropped by the network interface or its driver.  */
+  const char *devicename; /** name of the network interface */
+} NethogsPackageStats;
+
 /**
  * @brief Defines a callback to handle updates about applications
  * @param action NETHOGS_APP_ACTION_SET if data is being added or updated,
@@ -62,8 +73,7 @@ typedef void (*NethogsMonitorCallback)(int action,
  */
 
 NETHOGS_DSO_VISIBLE int nethogsmonitor_loop(NethogsMonitorCallback cb,
-                                            char *filter,
-                                            int to_ms);
+                                            char *filter, int to_ms);
 
 /**
  * @brief Enter the process monitoring loop and reports updates using the
@@ -87,14 +97,28 @@ NETHOGS_DSO_VISIBLE int nethogsmonitor_loop(NethogsMonitorCallback cb,
 NETHOGS_DSO_VISIBLE int nethogsmonitor_loop_devices(NethogsMonitorCallback cb,
                                                     char *filter, int devc,
                                                     char **devicenames,
-                                                    bool all,
-                                                    int to_ms);
+                                                    bool all, int to_ms);
 
 /**
  * @brief Makes the call to nethogsmonitor_loop return.
  */
 NETHOGS_DSO_VISIBLE void nethogsmonitor_breakloop();
 
+/**
+ * @brief returns the pcap packet stats per device
+ * 
+ * @param stats C-Style array the will hold the stats
+ * @param stats_size elements and therefore devices in stats
+ */
+NETHOGS_DSO_VISIBLE void nethogs_packet_stats(NethogsPackageStats **stats, int *stats_size);
+
+/**
+ * @brief Enables or disables the UDP recording. Default is False.
+ * 
+ * @param state state to set
+*/
+NETHOGS_DSO_VISIBLE void nethogs_enable_udp(bool state);
+
 #undef NETHOGS_DSO_VISIBLE
 #undef NETHOGS_DSO_HIDDEN
 
